<?php

namespace Vialibri\HarvestPlugin\CustomPostTypes;

class ProductData {

  public static function idList(array $ids) {

    $query = array_merge(self::getBasicQuery(), array(
      'fields' => 'ids', // Return the ID only, rather than the full object.
      'posts_per_page' => -1 // Turn off paging so that we get all products.
    ));

    return (new \WP_Query($query))->posts;
  }

  public static function listBooks(array $books, $pageNumber, $pageSize = 100) {
    $options = get_option('vharvest_cpt_settings');

    $query = array_merge(self::getBasicQuery(), array(
      'posts_per_page' => $pageSize,
      'paged' => $pageNumber,
      'order' => 'DESC',
      // Most recently modified first. But modified alone doesn't give consistent ordering when
      // there are many items with the same modified date. So we add ID too.
      'orderby' => 'modified ID'
    ));

    $q = new \WP_Query($query);

    $books = array();

    foreach ($q->posts as $bookPost) {

      $mapped = self::getMappedBookValues($options, $bookPost);

      $books[] = array_merge($mapped, array(
        'date_update' => $bookPost->post_modified, // Already in the right format.
        'source_id' => $bookPost->ID,
        'item_url' => get_permalink($bookPost),

        // Use SKU if available, and fall back to ID if there's no SKU.
        'sku_dealer_item_id' => !empty($mapped['sku_dealer_item_id'])
                                  ? $mapped['sku_dealer_item_id']
                                  : $bookPost->ID,
      ));
    }

    return $books;
  }

  private static function getBasicQuery() {

    $options = get_option('vharvest_cpt_settings');

    $query = array(
      'post_type' => $options['post_type'],
      'post_status' => 'publish', // Only products currently for sale.
      'tax_query' => array()
    );

    return $query;
  }

  private static function getMappedBookValues($options, $bookPost) {
    $fields = SettingsTab::getFields();
    $values = array();

    foreach ($fields as $key => $details) {
      $mapping = $options['mapping_'.$key];
      $subfields = json_decode($options['mapping_'.$key.'_subfields'] ?? '[]');
      $values[$key] = self::getMappedFieldValue($mapping, $subfields, $details['type'], $bookPost);
    }

    return $values;
  }

  private static function getMappedFieldValue($mapping, $subfields, $type, $bookPost) {
    if(empty($mapping)) {
      return null;
    }

    if ($mapping === 'multiple') {
      return implode(' ', array_filter(array_map(
        function($subfield) use ($type, $bookPost) {
          return self::getMappedFieldValue($subfield, [], $type, $bookPost);
        },
        $subfields
      )));
    } else {
      $value = null;
      list($mappingType, $mappingValue) = explode(':', $mapping);

      if($mappingType === 'post') {
        switch($mappingValue) {
          case 'title':
            $value = $bookPost->post_title;
            break;
          case 'content':
            $value = $bookPost->post_content;
            break;
          case 'excerpt':
            $value = $bookPost->post_excerpt;
            break;
          case 'featured_image':
            $mediaId = get_post_thumbnail_id($bookPost);
            if(!empty($mediaId)) {
              $value = wp_get_attachment_url($mediaId);
            }
            break;
        }
      } elseif($mappingType === 'meta') {
        $value = get_post_meta($bookPost->ID, $mappingValue, true);
      } elseif($mappingType === 'acf' && class_exists('ACF')) {
        $value = get_field($mappingValue, $bookPost->ID);
      }

      if($type === 'text') {
        $value = trim(wp_strip_all_tags($value));
      } elseif ($type === 'media') {
        if(preg_match('/^[0-9]*$/', $value)) {
          // Value is an integer. We assume it's a media ID.
          $value = wp_get_attachment_url($value);
        }
        if(empty($value)) {
          $value = null;
        }
      }

      return $value;
    }
  }

}
